/*
 * @author herbert
 * @date 04.08.2006
 * @version $Id: WriteablePortalConnectorExample.java,v 1.2 2006-08-28 14:55:39 herbert Exp $
 */
package com.gentics.portal.node.connector.examples;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import org.apache.log4j.PropertyConfigurator;

import com.gentics.api.lib.datasource.DatasourceInfo;
import com.gentics.api.lib.datasource.WriteableDatasource;
import com.gentics.api.lib.resolving.Changeable;
import com.gentics.api.lib.resolving.Resolvable;
import com.gentics.api.portalnode.connector.PortalConnectorFactory;

/**
 * A simple Writeable PortalConnector example which 
 * retrieves a writeable datasource and shows how to
 * create and change content objects.
 * @author herbert
 */
public class WriteablePortalConnectorExample {

    /**
     * @param args
     */
    public static void main(String[] args) {
        // Initialize log4j
        PropertyConfigurator.configure(WriteablePortalConnectorExample.class.getResource("logging.properties"));
        try {
            // First, retrieve the Writeable Datasource.
            WriteableDatasource ds = getDatasource();

            // Create a new object, by first putting all attributes into a hash map..
            Map newFile = new HashMap();
            newFile.put("obj_type", "10008"); // Create a file.
            
            // We want to create a new file so load some binary data...
            InputStream input = WriteablePortalConnectorExample.class
                    .getResourceAsStream("WriteablePortalConnectorExample.class");
            ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
            byte[] buf = new byte[1024];
            int c;
            while ((c = input.read(buf)) > 0) {
                outputStream.write(buf, 0, c);
            }
            
            // Since we now got the binary data, write it as new attribute.
            newFile.put("binarycontent", outputStream.toByteArray());
            newFile.put("name", "Sample Class File");
            newFile.put("filename", "WriteablePortalConnectorExample.class");
            // Now we have all attributes set, so create the actual Changeable. (Content Object)
            Changeable changeable = ds.create(newFile);
            
            // We now have a Changeable (Content Object) - So insert it into the datasource.
            DatasourceInfo info = ds.insert(Collections.singleton(changeable));

            // After inserting the new resolvable, it got assigned a new unique contentid.
            Iterator i = info.getAffectedRecords().iterator();
            Resolvable changed = (Resolvable) i.next();
            String contentid = (String) changed.get("contentid");
            
            // We got the contentid.
            System.out.println("New contentid: " + contentid);
            
            
            // Now retrieving the object again to make some changes.
            Changeable newChangeable = PortalConnectorFactory.getChangeableContentObject( contentid, ds );
            newChangeable.setProperty("name", "This is a new name");
            
            // Now save the changes
            ds.update(Collections.singleton(newChangeable));
            
            
            // and.. finally delete our object again.
            ds.delete(Collections.singleton(changed));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Creates the Writeable Datasource.
     * @return datasource
     * @throws Exception
     */
    protected static WriteableDatasource getDatasource() throws Exception {
        Properties handleProperties = new Properties();
        // Load the properties for the datasource connection from the
        // filesystem.
        handleProperties.load(AdvancedPortalConnectorExample.class
                .getResourceAsStream("jdbc-handle.properties"));

        return PortalConnectorFactory.createWriteableDatasource(handleProperties);
    }
}
